/**
 * Copyright (C) 2010  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.identity.auth;

import javax.security.auth.login.LoginException;

import org.ow2.bonita.util.Misc;

/**
 * 
 * @author Elias Ricken de Medeiros
 *
 */
public class BonitaRESTLoginModule extends BonitaRemoteLoginModule {

	
  /**
   * Method to commit the authentication process (phase 2). This method is
   * called if the LoginContext's overall authentication succeeded (the relevant
   * REQUIRED, REQUISITE, SUFFICIENT and OPTIONAL LoginModules succeeded). If
   * this LoginModule's own authentication attempt succeeded (checked by
   * retrieving the private state saved by the login method), then this method
   * associates relevant Principals and Credentials with the Subject located in
   * the LoginModule. If this LoginModule's own authentication attempted failed,
   * then this method removes/destroys any state that was originally saved.
   * 
   * @return true if this method succeeded, or false if this LoginModule should
   *         be ignored.
   * @throws LoginException
   *           if the commit fails
   */
  public boolean commit() throws LoginException {
    super.commit();
    PasswordOwner.setPassword(Misc.hash((String)this.sharedState.get(JAVAX_SECURITY_AUTH_LOGIN_PASSWORD)));
    return true;
  }

  /**
   * Method to abort the authentication process (phase 2). This method is called
   * if the LoginContext's overall authentication failed. (the relevant
   * REQUIRED, REQUISITE, SUFFICIENT and OPTIONAL LoginModules did not succeed).
   * If this LoginModule's own authentication attempt succeeded (checked by
   * retrieving the private state saved by the login method), then this method
   * cleans up any state that was originally saved.
   * 
   * @return true if this method succeeded, or false if this LoginModule should
   *         be ignored.
   * @throws LoginException
   *           if the abort fails
   */
  public boolean abort() throws LoginException {
    super.abort();
    PasswordOwner.setPassword(null);
    return true;
  }

  /**
   * Method which logs out a Subject. An implementation of this method might
   * remove/destroy a Subject's Principals and Credentials.
   * 
   * @return true if this method succeeded, or false if this LoginModule should
   *         be ignored.
   * @throws LoginException
   *           if the logout fails
   */
  public boolean logout() throws LoginException {
    super.logout();
    PasswordOwner.setPassword(null);
    return true;
  }


}
