/**
 * Copyright (C) 2010  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.runtime.event;

import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.transaction.Synchronization;

import org.ow2.bonita.env.Environment;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.runtime.model.Execution;
import org.ow2.bonita.services.CommandService;
import org.ow2.bonita.util.Command;
import org.ow2.bonita.util.EnvTool;

public class AbortProcess implements Synchronization {

  static final Logger LOG = Logger.getLogger(SendError.class.getName());

  private CommandService commandService;
  private ProcessInstanceUUID instanceUUID;

  public AbortProcess(final CommandService commandService, final ProcessInstanceUUID instanceUUID) {
    this.commandService = commandService;
    this.instanceUUID = instanceUUID;
  }

  public void afterCompletion(int arg0) {
    if (LOG.isLoggable(Level.FINE)) {
      LOG.fine("Aborting process instance: " + instanceUUID.getValue());
    }
    commandService.execute(new AbortProcessCommand(instanceUUID));
  }

  public void beforeCompletion() {
  }

  public String toString() {
    return "AbortProcess";
  }

  private static class AbortProcessCommand implements Command<Void> {

    private static final long serialVersionUID = -4923924149254428182L;
    private ProcessInstanceUUID instanceUUID;

    public AbortProcessCommand(ProcessInstanceUUID instanceUUID) {
      super();
      this.instanceUUID = instanceUUID;
    }

    public Void execute(Environment environment) throws Exception {
      Set<Execution> executions = EnvTool.getJournal().getExecutions(instanceUUID);
      executions.iterator().next().abort();
      return null;
    }
  }

}