/**
 * Copyright (C) 2010  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.runtime.event;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.transaction.Synchronization;

import org.ow2.bonita.env.Environment;
import org.ow2.bonita.services.CommandService;
import org.ow2.bonita.util.Command;
import org.ow2.bonita.util.EnvTool;

public class SendError implements Synchronization {

  static final Logger LOG = Logger.getLogger(SendError.class.getName());

  private CommandService commandService;
  private OutgoingEventInstance outgoingEvent;

  public SendError(final CommandService commandService, final OutgoingEventInstance outgoingEvent) {
    this.commandService = commandService;
    this.outgoingEvent = outgoingEvent;
  }

  public void afterCompletion(int arg0) {
    if (LOG.isLoggable(Level.FINE)) {
      LOG.fine("Adding outgoing evnet");
    }
    commandService.execute(new MyCommand(outgoingEvent));
  }

  public void beforeCompletion() {
  }

  public String toString() {
    return "SendError";
  }

  private static class MyCommand implements Command<Void> {

    private static final long serialVersionUID = -4923924149254428182L;
    private OutgoingEventInstance outgoingEvent;

    public MyCommand(OutgoingEventInstance outgoingEvent) {
      super();
      this.outgoingEvent = outgoingEvent;
    }

    public Void execute(Environment environment) throws Exception {
      EnvTool.getEventService().fire(outgoingEvent);
      return null;
    }
  }

}